/*
 * led-power - io-avr.c
 *
 * Copyright (C) 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <avr/io.h>
#include <io.h>
#include <ltypes.h>
#include <adc.h>
#include <compiler.h>

#define PINB_MOSI	(BIT(PB0))
#define PINB_MISO	(BIT(PB1))
#define PINB_EN		(BIT(PB3))

#define ADC_VIN			1
#define ADC_VOUT		2

#define R1		750.0
#define R2		5.1e3
#define VREF		5.0
#define ADC_RANGE	((R1 + R2) / R1 * VREF)
#define C_SHIFT		0
#define IN_SCALER	(u16)((ADC_RANGE / 65.536) * (64UL << C_SHIFT))

static unsigned int scale_adc(unsigned int x)
{
	return (IN_SCALER * x) >> C_SHIFT;
}

unsigned int vin_get(void)
{
	return scale_adc(adc_get(ADC_VIN));
}

unsigned int vout_get(void)
{
	return scale_adc(adc_get(ADC_VOUT));
}

void switch_set(unsigned char enabled)
{
	if (enabled)
		PORTB |= PINB_EN;
	else
		PORTB &= ~PINB_EN;
}

#if 1
void logger(unsigned int vin, unsigned int vout, unsigned int ic,
		unsigned int p, unsigned int temp, unsigned int enabled)
{
}
#endif

void io_xmit(unsigned char enabled)
{
	if (enabled)
		PORTB |= PINB_MISO;
	else
		PORTB &= ~PINB_MISO;
}

void io_init(void)
{
	/* just to be sure */
	DDRB = 0;
	PORTB = 0;
	DDRB = PINB_EN | PINB_MISO;

	/* Disable Analog Comparator to save power */
	ACSR &= BIT(ACIE);
	ACSR |= BIT(ACD);
}
