/*
 * ar4c-dcdc - sim-posix.c
 *
 * Simulator for POSIX
 *
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <compiler.h>
#include <ltypes.h>
#include <stdio.h>
#include <io.h>

static unsigned int sim_vin = 30219; 
static unsigned int sim_vout = 0; // 22700; 

unsigned int vin_get(void)
{
	return sim_vin;
}

unsigned int vout_get(void)
{
	return sim_vout;
}

/*
 * i = C du/dt
 *
 * C = 17.6 mF, dt = 1/61.035
 *
 * du = 1/C dt * i
 * 1/C * dt = 0.93 \approx 1
 */
void switch_set(unsigned char enabled)
{
	if (enabled) {
		unsigned int ic = 0;

		if (sim_vin > sim_vout) {
			unsigned int delta = sim_vin - sim_vout;

			if (delta >= 1230)
				ic = 1059;
			else if (delta >= 300)
				ic = (delta - 300) * 5 / 4;
		}

		sim_vout += ic; 
	}

	if (sim_vout < 300)
		sim_vout = 0;
	else
		sim_vout -= 300;
}

void io_xmit(unsigned char enabled)
{
}

void logger(unsigned int vin, unsigned int vout, unsigned int ic,
		unsigned int p, unsigned int temp, unsigned int enabled)
{
	printf("%d	%d	%d	%d	%d	%d\n",
			vin, vout, ic, p, temp, enabled);
}

void adc_init(void)
{
}

void io_init(void)
{
}

void ee_write(u8 x, unsigned int addr)
{
}

u8 ee_read(unsigned int addr)
{
	return 0;
}
